package org.processmining.plugins.log.ui.logoverview;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.CancellationException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.deckfour.xes.model.XLog;
import org.processmining.plugins.log.ui.logoverview.filter.Filter;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class FilterDialog extends JDialog {
	private static final long serialVersionUID = 1L;

	public static Filter showAddFilterDialog(JPanel parent, XLog log, List<Filter> availableFilters) {
		Filter result = null;
		FilterDialog dialog = null;
		
		try {
			dialog = new FilterDialog(log, availableFilters);
		}
		catch(NullPointerException e) {
			CancellationException cancel = new CancellationException();
			
			cancel.initCause(e);
			
			throw cancel;
		}
		
		dialog.setLocationRelativeTo(parent);
		
		try {
			dialog.setVisible(true);
			result = dialog.getResult();
		}
		finally {
			dialog.dispose();
		}
		
		if(result == null) {
			throw new CancellationException();
		}
		
		return result;
	}
	
	private Filter result = null;
	private XLog log = null;
	private JComboBox comboboxFilters;
	private JTextArea textDescription;
	
	public FilterDialog(XLog log, List<Filter> availableFilters) {
		
		if(log == null) {
			throw new NullPointerException();
		}
		
		this.log = log;
		
		SlickerFactory factory = SlickerFactory.instance();
		
		//Configure the dialog.
		setModalityType(ModalityType.APPLICATION_MODAL);
		
		//Create the components.
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		add(panel);
		
		JPanel northpanel = new JPanel();
		panel.add(northpanel, BorderLayout.NORTH);
		
		//Create filter selection combobox.
		Collections.sort(availableFilters, new Comparator<Filter>() {
			public int compare(Filter filter0, Filter filter1) {
				return filter0.getName().compareTo(filter1.getName());
			}
		});
		FilterEntry[] filters = new FilterEntry[availableFilters.size()];
		for(int i = 0; i < availableFilters.size(); i++) {
			filters[i] = new FilterEntry(availableFilters.get(i));
		}
		comboboxFilters = factory.createComboBox(filters);
		comboboxFilters.setPreferredSize(new Dimension(220, 24));
		northpanel.add(comboboxFilters);
		
		//Create filter configuration button.
		final JButton buttonConfigure = factory.createButton("Configure");
		northpanel.add(buttonConfigure);
		
		//Create filter description label.
		textDescription = new JTextArea();
		textDescription.setPreferredSize(new Dimension(250, 40));
		textDescription.setLineWrap(true);
		panel.add(textDescription, BorderLayout.CENTER);
		
		//Create confirmation button.
		final JButton buttonAdd = factory.createButton("Add");
		panel.add(buttonAdd, BorderLayout.SOUTH);
		
		comboboxFilters.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent ev) {
				
				buttonConfigure.setEnabled(((FilterEntry)((JComboBox)ev.getSource()).getSelectedItem()).getFilter().isConfigurable());
				updateDescription();
			}
		});
		
		buttonConfigure.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent ev) {
				
				try {
					
					((FilterEntry)comboboxFilters.getSelectedItem()).getFilter().configure(FilterDialog.this.log);
					updateDescription();
				}
				catch(CancellationException e) {}
				catch(RuntimeException e) {
					
					System.err.println("An error occurred during the configuration of the selected filter.");
					e.printStackTrace();
					
					JOptionPane.showMessageDialog(FilterDialog.this, "An error occurred during the configuration of this filter. Please see the console message for more information.");
				}
			}
		});
		
		buttonAdd.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				
				result = ((FilterEntry)comboboxFilters.getSelectedItem()).getFilter();
				close();
			}
		});
		
		updateDescription();
		
		pack();
	}
	
	private void updateDescription() {
		textDescription.setText(((FilterEntry)comboboxFilters.getSelectedItem()).getFilter().getDescription());
	}
	
	private void close() {
		setVisible(false);
	}
	
	public Filter getResult() {
		return result;
	}
	
	private static class FilterEntry {
		private Filter filter;
		
		public FilterEntry(Filter filter) {
			this.filter = filter;
		}
		
		public Filter getFilter() {
			return filter;
		}
		
		public String toString() {
			return filter.getName();
		}
	}
}
