package org.processmining.plugins.log.ui.logoverview;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.concurrent.CancellationException;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.log.ui.logoverview.filter.Filter;
import org.processmining.plugins.log.ui.logoverview.filter.FilterLog;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class FilterInteractionPanel extends JPanel implements ViewInteractionPanel {
	private static final long serialVersionUID = 1L;
	
	private LogOverviewVisualizer panel;
	private FilterLog log;
	private FilterListModel filters;
	private int selectedDataSet = 0;
	
	public FilterInteractionPanel(final LogOverviewVisualizer panel, FilterLog log) {
		
		if(panel == null) {
			throw new IllegalArgumentException("The provided panel is not an instance of LogOverviewVisualizer.");
		}
		
		this.panel = panel;
		
		if(log == null) {
			throw new IllegalArgumentException("The FilterLog instance cannot be 'null'.");
		}
		
		this.log = log;
		
		//Prepare initial data.
		filters = new FilterListModel();
		
		//Set properties of the main panel.
		setPreferredSize(new Dimension(400, 300));
		double[][] tableLayoutSize = new double[][] { {0.3, 0.2, 0.3, TableLayout.FILL}, {24, 24, 24, 24, 24, 24, TableLayout.FILL, 24} };
		setLayout(new TableLayout(tableLayoutSize));
		
		SlickerFactory sf = SlickerFactory.instance();
		
		//Create GUI components
		final JList listFilters = new JList(filters);
		listFilters.setCellRenderer(new FilterListCellRenderer(this));
		add(new JScrollPane(listFilters), "0, 0, 2, 6");
		
		final JButton buttonAdd = sf.createButton("Add");
		buttonAdd.setToolTipText("Add a new filter.");
		add(buttonAdd, "0, 7");
		
		final JButton buttonRemove = sf.createButton("Remove");
		buttonRemove.setEnabled(false);
		buttonRemove.setToolTipText("Remove the selected filter.");
		add(buttonRemove, "2, 7");
		
		final JButton buttonMoveUp = sf.createButton("Move up");
		buttonMoveUp.setEnabled(false);
		buttonMoveUp.setToolTipText("Move the selected filter up.");
		add(buttonMoveUp, "3, 4");
		
		final JButton buttonMoveDown = sf.createButton("Move down");
		buttonMoveDown.setEnabled(false);
		buttonMoveDown.setToolTipText("Move the selected filter down.");
		add(buttonMoveDown, "3, 5");
		
		final JButton buttonSelect = sf.createButton("Select");
		buttonSelect.setEnabled(false);
		buttonSelect.setToolTipText("Activate the selected result.");
		add(buttonSelect, "3, 0");
		
		final JButton buttonConfigure = sf.createButton("Configure");
		buttonConfigure.setEnabled(false);
		buttonConfigure.setToolTipText("Configure the selected filter.");
		add(buttonConfigure, "3, 1");
		
		final JButton buttonExport = sf.createButton("Export");
		buttonExport.setEnabled(true);
		buttonExport.setToolTipText("Export the event log in its current state.");
		add(buttonExport, "3, 2");
		
		listFilters.addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent e) {
				Object selection = listFilters.getSelectedValue();
				
				if(listFilters.getSelectedIndex() == 0) {
					buttonConfigure.setEnabled(true);
					buttonRemove.setEnabled(false);
					buttonSelect.setEnabled(true);
				}
				else if(selection instanceof Filter) {
					Filter filter = (Filter)selection;
					
					buttonConfigure.setEnabled(filter.isConfigurable());
					buttonRemove.setEnabled(true);
					buttonSelect.setEnabled(false);
					buttonMoveUp.setEnabled(true);
					buttonMoveDown.setEnabled(true);
				}
				else {
					buttonConfigure.setEnabled(false);
					buttonRemove.setEnabled(false);
					buttonSelect.setEnabled(true);
					buttonMoveUp.setEnabled(false);
					buttonMoveDown.setEnabled(false);
				}
			}
		});
		listFilters.addMouseListener(new MouseListener() {

			@Override
			public void mouseClicked(MouseEvent ev) {
				JList list = (JList)ev.getSource();
				
				int index = list.locationToIndex(ev.getPoint());
				if(ev.getClickCount() == 2) {
					
					if(index >= 0 && index % 2 == 0) {
						actionSelectDataSet(list, index);
					}
					else if(index > 0 && index % 2 == 1) {
						actionConfigureFilter(listFilters, (Filter)filters.getElementAt(index));
					}
				}
			}

			@Override
			public void mouseEntered(MouseEvent ev) {}

			@Override
			public void mouseExited(MouseEvent ev) {}

			@Override
			public void mousePressed(MouseEvent ev) {}

			@Override
			public void mouseReleased(MouseEvent ev) {}
		});
		
		buttonAdd.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent ev) {
				
				try {
					filters.addFilter(FilterDialog.showAddFilterDialog(FilterInteractionPanel.this, FilterInteractionPanel.this.log, panel.findAvailableFilters()));
				}
				catch(CancellationException e) {
					System.err.println("Add filter dialog cancelled.");
					
					if(e.getCause() != null) {
						e.getCause().printStackTrace();
					}
				}
			}
		});
		
		buttonConfigure.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Object selection = listFilters.getSelectedValue();
				
				if(listFilters.getSelectedIndex() == 0) {
					String tagName = JOptionPane.showInputDialog("Enter tag name: ");
					
					if(tagName != null) {
						
						if(tagName.isEmpty()) {
							filters.setInitialDataSet(null);
						}
						else {
							SortedSet<EID> events = new TreeSet<EID>();
							
							for(Map.Entry<String, SortedSet<EID>> entry : FilterInteractionPanel.this.panel.getTagOriginals(tagName).entrySet()) {
								events.addAll(entry.getValue());
							}
							
							filters.setInitialDataSet(events);
						}
						
						refreshDataSet();
					}
				}
				else if(selection instanceof Filter) {
					actionConfigureFilter(listFilters, (Filter)selection);
				}
			}
		});
		
		buttonRemove.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent ev) {
				
				try {
					int index = listFilters.getSelectedIndex();
					filters.removeFilter(index);
					
					//Update data set: make sure that either the same or the data set
					// above is selected, so that we don't unexpectedly turn on
					// additional filters.
					if(selectedDataSet > index) {
						selectDataSet(selectedDataSet-2);
					}
				}
				catch(IllegalArgumentException e) {
					JOptionPane.showMessageDialog(FilterInteractionPanel.this, e.getMessage());
				}
			}
		});
		
		buttonSelect.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				int index = listFilters.getSelectedIndex();
				
				if(index % 2 == 0) {
					actionSelectDataSet(listFilters, index);
				}
			}
		});
		
		buttonExport.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent ev) {
				XConceptExtension conceptext = XConceptExtension.instance();
				String name = conceptext.extractName(FilterInteractionPanel.this.log);
				
				if(name == null) {
					name = "Filtered log";
				}
				else {
					name += " filtered";
				}
				
				//Query the user for a name for the exported log.
				name = JOptionPane.showInputDialog(FilterInteractionPanel.this, "Enter a name for the log: ", name);
				
				//If the user cancelled the input dialog, stop execution.
				if(name == null) {
					return;
				}
				
				//Generate a clean XLog instance.
				XLog exported = FilterInteractionPanel.this.log.generateXLogInstance();
				
				//Assign a name to the XLog instance.
				conceptext.assignName(exported, name);
				exported.getExtensions().add(conceptext);
				
				//Export the XLog instance.
				FilterInteractionPanel.this.panel.exportToFramework(exported, name);
				
				//Inform the user that the export is completed.
				JOptionPane.showMessageDialog(FilterInteractionPanel.this, "The event log has been exported to the ProM Framework.");
			}
		});
		
		buttonMoveUp.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				FilterListModel listmodel = ((FilterListModel)listFilters.getModel());
				int filterIndex = listFilters.getSelectedIndex();
				
				Filter filter = (Filter)listmodel.getElementAt(filterIndex);
				
				filters.moveFilter(filter, -1);
				
				refreshDataSet();
			}
		});
		
		buttonMoveDown.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				FilterListModel listmodel = ((FilterListModel)listFilters.getModel());
				int filterIndex = listFilters.getSelectedIndex();
				
				Filter filter = (Filter)listmodel.getElementAt(filterIndex);
				
				filters.moveFilter(filter, 1);
				
				refreshDataSet();
			}
		});
	}
	
	private void actionConfigureFilter(JList list, Filter filter) {
		
		if(!filter.isConfigurable()) {
			
			JOptionPane.showMessageDialog(null, "This filter can not be configured.");
			return;
		}
		
		try {
			filter.configure(log);
		
			if(filters.getFilters(selectedDataSet).contains(filter)) {
				//Reapplying filters because the configuration has changed.
				refreshDataSet();
			}
		}
		catch(CancellationException e) {
			//TODO debugging info
			System.err.println("Configuration cancelled.");
		}
		
		list.repaint();
	}
	
	private void actionSelectDataSet(JList list, int index) {
		
		selectDataSet(index);
		
		list.repaint();
	}
	
	/**
	 * Refresh the current data set.
	 */
	public void refreshDataSet() {
		selectDataSet(selectedDataSet);
	}
	
	//TODO if the selected data set (index) is higher than the current, only apply missing filters.
	/**
	 * Select a new data set from the Filters list.
	 */
	public void selectDataSet(int index) {
		selectedDataSet = index;
		
		//If a specific initial data set has been specified, use this set.
		SortedSet<EID> initialSet = filters.getInitialDataSet();
		if(initialSet != null) {
			
			try {
				log.setRemainingEvents(initialSet);
			}
			catch(IllegalStateException e) {
				System.err.println("The initial event log state defined by the specified set of events is invalid. If, after applying the provided filters, this state is still invalid then we will reset the event log to its pristine state.");
			}
		}
		//Otherwise reset the log and use the complete log.
		else {
			log.reset();
		}
		
		//Apply filters.
		List<Filter> filters = this.filters.getFilters(selectedDataSet);
		for(Filter filter : filters) {
			
			try {
				filter.filter(log);
			}
			catch(RuntimeException e) {
				
				System.err.println("An error occurred during the execution of filter '"+filter.getName()+"' ("+filter.getDescription()+"):");
				e.printStackTrace();
				
				JOptionPane.showMessageDialog(FilterInteractionPanel.this, "An error has occurred during the execution of filter '"+filter.getName()+"'\n("+filter.getDescription()+").\nThe resulting event log may be incorrect.");
			}
		}
		
		try {
			//Remove empty traces from the list.
			log.cleanUp();
		}
		catch(IllegalStateException e) {
			
			System.err.println("The resulting event log is in a state that is not allowed according to the XES standard. The event log will be reset to the pristine state.");
			log.reset();
			
			JOptionPane.showMessageDialog(this, "The resulting event log is in an invalid state according to the XES standard.\nThe event log is reset to its original state.");
		}
		
		//Notify interactive views that an update is required.
		panel.updateInteractiveViews(true, false);
	}
	
	public int getSelectedDataSet() {
		return selectedDataSet;
	}
	
	@Override
	public void updated() {}

	@Override
	public String getPanelName() {
		return "Filtering";
	}

	@Override
	public JComponent getComponent() {
		return this;
	}

	@Override
	public void setScalableComponent(ScalableComponent scalable) {}

	@Override
	public void setParent(ScalableViewPanel viewPanel) {}

	@Override
	public double getHeightInView() {
		return getPreferredSize().getHeight();
	}

	@Override
	public double getWidthInView() {
		return getPreferredSize().getWidth();
	}

	@Override
	public void willChangeVisibility(boolean to) {}
}
