package org.processmining.plugins.log.ui.logoverview;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.util.SortedSet;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListCellRenderer;

import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.log.ui.logoverview.filter.Filter;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class FilterListCellRenderer extends JPanel implements ListCellRenderer {
	private static final long serialVersionUID = 1L;

	private static final String LOCATION_ICON_FILTER = "resources/filter.png";
	
	private FilterInteractionPanel parent;
	private JLabel labelName;
	private JLabel labelDescription;
	private JRadioButton radioButton = null;
	private JLabel labelFilterImage = null;
	private SlickerFactory sf = SlickerFactory.instance();
	
	public FilterListCellRenderer(FilterInteractionPanel parent) {
		
		this.parent = parent;
		
		double[][] tableLayoutSize = new double[][] {{5, 30, TableLayout.FILL, 5}, {5, 14, 14, 5}};
		setLayout(new TableLayout(tableLayoutSize));
		
		labelName = sf.createLabel("Name");
		add(labelName, "2, 1");
		labelDescription = sf.createLabel("Description:");
		add(labelDescription, "2, 2");
		
		//prepare GUI elements for later use.
		radioButton = sf.createRadioButton("");
		labelFilterImage = new JLabel(new ImageIcon(this.getToolkit().getImage(LOCATION_ICON_FILTER)));
	}
	
	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {
		
		deleteDynamicComponents();
		
		if(index == 0) {
			
			labelName.setText("Initial data set");
			
			if(value instanceof SortedSet<?>) {
				@SuppressWarnings("unchecked")
				SortedSet<EID> initial = (SortedSet<EID>)value;
				
				labelDescription.setText("Dataset with "+initial.size()+" items");
			}
			else {
				labelDescription.setText("Using original log");
			}
			
			addRadioButton(index == parent.getSelectedDataSet());
		}
		else if(value instanceof Filter) {
			Filter filter = (Filter)value;
			
			labelName.setText(filter.getName());
			labelDescription.setText(filter.getDescription());
			
			addFilterImage();
		}
		else {
			labelName.setText("Result");
			labelDescription.setText("Result of filter");
			
			addRadioButton(index == parent.getSelectedDataSet());
		}
		
		setBackground(isSelected ? Color.YELLOW : Color.WHITE);
		
		return this;
	}
	
	private void deleteDynamicComponents() {
		remove(radioButton);
		remove(labelFilterImage);
	}
	
	private void addRadioButton(boolean selected) {
		
		add(radioButton, "1, 1, 1, 2");
		
		//The radio buttons are actually "fake", since we
		// don't put them in a group and we simply set
		// selected: true/false manually. This is because
		// we lose original functionality because they're
		// put inside a list.
		radioButton.setSelected(selected);
	}
	
	private void addFilterImage() {
		add(labelFilterImage, "1, 1, 1, 2");
	}
}
