package org.processmining.plugins.log.ui.logoverview;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class LogOverviewInteractionPanel extends JPanel implements ViewInteractionPanel {
	private static final long serialVersionUID = 1L;
	
	private LogOverviewVisualizer panel;
	
	private JComboBox comboboxCurrentView;
	private JTextField textNewTagName;
	private JButton buttonCreateNewTag;
	private JButton buttonStoreVisible;
	private JList listTags;
	private TagsListModel tagsListModel;
	
	public LogOverviewInteractionPanel(LogOverviewVisualizer panel, Map<InteractiveVisualization, String> views) {
		
		if(panel == null) {
			throw new IllegalArgumentException("The provided panel is not an instance of LogOverviewVisualizer.");
		}
		
		this.panel = panel;
		
		if(views == null) {
			throw new IllegalArgumentException("The list of views parameter may not be 'null'.");
		}
		
		/*
		 * Configure the panel
		 */
		setPreferredSize(new Dimension(400, 500));
		double[][] tableLayoutSize = new double[][] {{10, 0.3, 0.2, 0.2, TableLayout.FILL, 10}, {10, 24, 24, 24, 24, 24, 24, TableLayout.FILL, 10}};
		setLayout(new TableLayout(tableLayoutSize));
		
		SlickerFactory factory = SlickerFactory.instance();
		
		/*
		 * Build the panel components
		 */
		add(factory.createLabel("Visualization: "), "1, 1");
		
		comboboxCurrentView = factory.createComboBox(ComboboxEntry.compileList(views));
		comboboxCurrentView.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent ev) {
				LogOverviewInteractionPanel.this.panel.selectView(((ComboboxEntry)((JComboBox)ev.getSource()).getSelectedItem()).view);
			}
		});
		comboboxCurrentView.setPreferredSize(new Dimension(350, 25));
		add(comboboxCurrentView, "2, 1, 4, 1");
		
		buttonStoreVisible = factory.createButton("Store visible elements");
		buttonStoreVisible.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent ev) {
				
				try {
					tagsListModel.setVisible(LogOverviewInteractionPanel.this.panel.getCurrentView().getVisibleElements());
				}
				catch(UnsupportedOperationException e) {
					JOptionPane.showMessageDialog(null, "The currently active visualization cannot transfer visible elements.");
				}
			}
		});
		add(buttonStoreVisible, "2, 4, 4, 4");
		
		//Action Listener for handling setting a new tag.
		ActionListener setTagActionListener = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent ev) {
				
				if(textNewTagName.getText().isEmpty()) {
					JOptionPane.showMessageDialog(null, "Please enter a name for the tag first.");
					return;
				}
				
				InteractiveVisualization iv = LogOverviewInteractionPanel.this.panel.getCurrentView();
				
				try {
					
					//Create the tag using the provided name as tag name.
					tagsListModel.setTag(textNewTagName.getText(), iv.getSelectedElements());
					
					//Clear the name once the tag is set and only if tagging succeeded.
					textNewTagName.setText("");
				}
				catch(UnsupportedOperationException e) {
					JOptionPane.showMessageDialog(null, "The currently active visualization cannot transfer selected elements.");
				}
			}
		};
		
		add(factory.createLabel("New tag: "), "1, 3");
		textNewTagName = new JTextField();
		textNewTagName.setPreferredSize(new Dimension(100, 20));
		textNewTagName.addActionListener(setTagActionListener);
		add(textNewTagName, "2, 3, 3, 3");
		
		buttonCreateNewTag = factory.createButton("Set");
		buttonCreateNewTag.addActionListener(setTagActionListener);
		add(buttonCreateNewTag, "4, 3");
		
		add(factory.createLabel("Tags: "), "1, 6");
		tagsListModel = new TagsListModel(panel);
		listTags = new JList(tagsListModel);
		listTags.setCellRenderer(new TagsListCellRenderer(panel));
		listTags.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseClicked(MouseEvent ev) {
				JList list = (JList)ev.getSource();
				
				int index = list.locationToIndex(ev.getPoint());
				if(index >= 0) {
					Object element = tagsListModel.getElementAt(index);
					
					if(element instanceof ListElementTag) {
						tagsListModel.setExpanded(((ListElementTag)element).getTagName(), !tagsListModel.isExpanded(((ListElementTag)element).getTagName()));
					}
					else if(element instanceof ListElementInstance) {
						LogOverviewInteractionPanel.this.panel.setTagInstanceEnabled(
								((ListElementInstance) element).getTagName(),
								((ListElementInstance) element).getInstanceName(),
								!LogOverviewInteractionPanel.this.panel.isTagEnabled(((ListElementInstance)element).getTagName(), ((ListElementInstance)element).getInstanceName()));
					}
					
					list.repaint();
				}
			}
			
			@Override
			public void mouseEntered(MouseEvent ev) {}
			
			@Override
			public void mouseExited(MouseEvent ev) {}
			
			@Override
			public void mousePressed(MouseEvent ev) {}
			
			@Override
			public void mouseReleased(MouseEvent ev) {}
		});
		JScrollPane scrollListTags = new JScrollPane(listTags);
		scrollListTags.setPreferredSize(new Dimension(200, 400));
		add(scrollListTags, "2, 6, 4, 7");
	}
	
	private static class ComboboxEntry {
		
		public static Object[] compileList(Map<InteractiveVisualization, String> views) {
			List<ComboboxEntry> list = new LinkedList<ComboboxEntry>();
			
			for(InteractiveVisualization view : views.keySet()) {
				list.add(new ComboboxEntry(views.get(view), view));
			}
			
			return list.toArray();
		}
		
		public final String name;
		public final InteractiveVisualization view;
		
		public ComboboxEntry(String name, InteractiveVisualization view) {
			this.name = name;
			this.view = view;
		}
		
		public String toString() {
			return name;
		}
	}
	
	/*
	 * Implementation of View Interaction Panel interface (non-Javadoc)
	 * @see org.processmining.framework.util.ui.scalableview.ScalableComponent.UpdateListener
	 */
	
	@Override
	public void updated() {
		
		tagsListModel.updated();
		
		listTags.repaint();
	}

	@Override
	public String getPanelName() {
		return "Overview";
	}

	@Override
	public JComponent getComponent() {
		return this;
	}

	@Override
	public void setScalableComponent(ScalableComponent scalable) {
		//The scalable component is not required for this Interaction Panel,
		// because this interaction panel depends on operations with the
		// InteractiveVisualization interface. Therefore it uses the
		// provided InteractivityManager to retrieve all the necessary
		// information.
	}

	@Override
	public void setParent(ScalableViewPanel viewPanel) {
		//The Scalable View Panel is not required for this Interaction Panel,
		// because this interaction panel depends on operations with the
		// InteractiveVisualization interface. Therefore it uses the
		// provided InteractivityManager to retrieve all the necessary
		// information.
	}

	@Override
	public double getHeightInView() {
		return getPreferredSize().getHeight();
	}

	@Override
	public double getWidthInView() {
		return getPreferredSize().getWidth();
	}

	@Override
	public void willChangeVisibility(boolean to) {}
}
