package org.processmining.plugins.log.ui.logoverview;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.SortedSet;

import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractivityManager;

public class TagsListModel implements ListModel {
	private InteractivityManager manager;
	private final Set<String> expandedTags = new HashSet<String>();
	private final List<ListDataListener> listeners = new ArrayList<ListDataListener>();
	
	public TagsListModel(InteractivityManager manager) {
		this.manager = manager;
	}
	
	public void updated() {
		updateListeners();
	}
	
	private void updateListeners() {
		
		for(ListDataListener listener : listeners) {
			listener.contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize()));
		}
	}
	
	public boolean isExpanded(String tag) {
		return expandedTags.contains(tag);
	}
	
	public void setExpanded(String tag, boolean expanded) {
		
		if(!expanded) {
			expandedTags.remove(tag);
		}
		else {
			expandedTags.add(tag);
		}
		
		updateListeners();
	}
	
	public int getSize() {
		int count = 0;
		
		for(String tag : manager.getTags()) {
			
			//the tag name
			count += 1;
			
			if(expandedTags.contains(tag)) {
				//all instances
				count += getTagSize(tag);
			}
		}
				
		return count;
	}
	
	private int getTagSize(String tag) {
		return manager.getInstances(tag).size();
	}
	
	public Object getElementAt(int index) {
		int current = -1;
		
		for(String tag : manager.getTags()) {
			current++;
			
			//Show the tag.
			if(index == current) {
				return new ListElementTag(tag);
			}
			
			//Only if a tag is expanded, show the instances.
			if(expandedTags.contains(tag)) {
				
				for(String instance : manager.getInstances(tag)) {
					current++;
					
					if(index == current) {
						return new ListElementInstance(tag, instance);
					}
				}
			}
		}
		
		throw new NoSuchElementException();
	}
	
	public void addListDataListener(ListDataListener l) {
		
		if(l == null) {
			throw new NullPointerException();
		}
		
		listeners.add(l);
	}
	
	public void removeListDataListener(ListDataListener l) {
		
		if(l == null) {
			throw new NullPointerException();
		}
		
		listeners.remove(l);
	}

	public void setTag(String text, Map<String, SortedSet<EID>> selectedElements) {
		
		manager.setTag(text, selectedElements);
		
		updateListeners();
	}

	public void setVisible(Map<String, SortedSet<EID>> visibleElements) {
		
		manager.setVisible(visibleElements);
		
		updateListeners();
	}
}
