package org.processmining.plugins.log.ui.logoverview.filter;

import java.util.concurrent.CancellationException;

import org.deckfour.xes.model.XLog;


public interface Filter {
	
	/**
	 * Name of the filter (just the name, independent of settings)
	 * 
	 * @return Returns the name of the filter.
	 */
	public String getName();
	
	/**
	 * Description of the filter.
	 * 
	 * A short description of the filter that can be shown in the list to allow for
	 * a user to distinguish between different filters. This description may contain
	 * some or all of the user's configuration parameters.
	 * 
	 * @return Returns a description of the filter.
	 */
	public String getDescription();
	
	/**
	 * Check whether the filter is configurable.
	 * 
	 * @return True if the filter is configurable, false otherwise.
	 */
	public boolean isConfigurable();
	
	/**
	 * A method for the purpose of calling the configuration dialog used to configure the filter.
	 * 
	 * @throws CancellationException Throws cancellation exception in case the configuration was aborted.
	 */
	public void configure(XLog log);
	
	/**
	 * Implementation of a filter (for the LogOverview plugin).
	 * 
	 * The Filter is run on the provided XLog instance (log) (which is the original log) and a set of Event Ids (EID) is returned.
	 *  
	 * @param log The original log to run the filter on.
	 * @return Returns the set of Event Ids (EIDs) that passed through the filter.
	 */
	public void filter(XLog log);
}
