package org.processmining.plugins.log.ui.logoverview.filter;

import java.util.Iterator;

import javax.swing.JOptionPane;

import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.plugins.log.ui.logoverview.annotations.LogOverviewAddon;

@KeepInProMCache
@LogOverviewAddon
public class RemoveContainingEvents implements Filter {
	private XAttribute condition = null;
	
	public RemoveContainingEvents() {
		condition = XFactoryRegistry.instance().currentDefault().createAttributeLiteral("concept:name", "x", null);
	}
	
	public String getName() {
		return "Remove events with the specified attribute";
	}
	
	public String getDescription() {
		return "Remove all events that contain an attribute: "+condition.getKey()+" = "+condition.toString();
	}
	
	public boolean isConfigurable() {
		return true;
	}
	
	public void configure(XLog log) {
		String key = JOptionPane.showInputDialog("Enter the name of the attribute", condition.getKey());
		
		if(key == null) {
			return;
		}
		
		String value = JOptionPane.showInputDialog("Enter the value for the attribute: ", condition.toString());
		
		if(value == null) {
			return;
		}
		
		condition = XFactoryRegistry.instance().currentDefault().createAttributeLiteral(key, value, null);
	}
	
	public void filter(XLog log) {
		
		Iterator<XTrace> traceIterator = log.iterator();
		while(traceIterator.hasNext()) {
			XTrace trace = traceIterator.next();
			
			Iterator<XEvent> eventIterator = trace.iterator();
			while(eventIterator.hasNext()) {
				XEvent event = eventIterator.next();
				
				if(condition.equals(event.getAttributes().get(condition.getKey()))) {
					eventIterator.remove();
				}
			}
		}
	}
}