package org.processmining.plugins.log.ui.logoverview.filter;

import java.util.concurrent.CancellationException;

import javax.swing.JOptionPane;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.plugins.log.ui.logoverview.annotations.LogOverviewAddon;

@KeepInProMCache
@LogOverviewAddon
public class RemoveFirstEventFilter implements Filter {
	private int number = 1;
	
	public RemoveFirstEventFilter() {
		this(1);
	}
	
	public RemoveFirstEventFilter(int number) {
		
		if(number < 0) {
			number = 0;
		}
		
		this.number = number;
	}
	
	@Override
	public String getName() {
		return "First event filter";
	}

	@Override
	public String getDescription() {
		return "Remove first "+number+" events from every trace.";
	}
	
	@Override
	public boolean isConfigurable() {
		return true;
	}
	
	@Override
	public void configure(XLog log) {
		String result = JOptionPane.showInputDialog(null, "Please enter number of events to remove: ", number);
		
		if(result == null) {
			throw new CancellationException();
		}
		
		try {
			number = Integer.parseInt(result);
		}
		catch(NumberFormatException e) {
			JOptionPane.showMessageDialog(null, "The number that was entered is invalid. No changes have been made.");
		}
	}
	
	@Override
	public void filter(XLog log) {
		
		for(XTrace tr : log) {
			FilterTrace trace = (FilterTrace)tr;
			
			for(int i = 0; i < number; i++) {
				
				try {
					trace.remove(0);
				}
				catch(IndexOutOfBoundsException e) {
					break;
				}
			}
		}
	}
}
